@@ -74,15 +74,21 @@ pub const TEEC_LOGIN_APPLICATION: u32       = 0x00000004;
 pub const TEEC_LOGIN_USER_APPLICATION: u32  = 0x00000005;
 pub const TEEC_LOGIN_GROUP_APPLICATION: u32 = 0x00000006;
 
+#[allow(non_camel_case_types)]
 pub type TEEC_Result = u32;
 
 #[repr(C)]
-pub struct TEEC_Context {
+pub struct TEEC_Context__Imp {
     pub fd: c_int,
     pub reg_mem: bool,
     pub memref_null: bool,
 }
 
+#[repr(C)]
+pub struct TEEC_Context {
+    pub imp: TEEC_Context__Imp,
+}
+
 #[repr(C)]
 pub struct TEEC_UUID {
     pub timeLow: u32,
@@ -92,15 +98,23 @@ pub struct TEEC_UUID {
 }
 
 #[repr(C)]
-pub struct TEEC_Session {
+pub struct TEEC_Session__Imp {
     pub ctx: *mut TEEC_Context,
     pub session_id: u32,
 }
 
 #[repr(C)]
-pub union SharedMemoryFlagsCompat {
-    dummy: bool,
-    flags: u8,
+pub struct TEEC_Session {
+    pub imp: TEEC_Session__Imp,
+}
+
+#[repr(C)]
+pub struct TEEC_SharedMemory__Imp {
+    pub id: c_int,
+    pub alloced_size: size_t,
+    pub shadow_buffer: *mut c_void,
+    pub registered_fd: c_int,
+    pub flags: u32,
 }
 
 #[repr(C)]
@@ -108,11 +122,7 @@ pub struct TEEC_SharedMemory {
     pub buffer: *mut c_void,
     pub size: size_t,
     pub flags: u32,
-    pub id: c_int,
-    pub alloced_size: size_t,
-    pub shadow_buffer: *mut c_void,
-    pub registered_fd: c_int,
-    pub internal: SharedMemoryFlagsCompat,
+    pub imp: TEEC_SharedMemory__Imp,
 }
 
 #[derive(Copy, Clone)]
@@ -145,12 +155,17 @@ pub union TEEC_Parameter {
     pub value: TEEC_Value,
 }
 
+#[repr(C)]
+pub struct TEEC_Operation__Imp {
+    pub session: *mut TEEC_Session,
+}
+
 #[repr(C)]
 pub struct TEEC_Operation {
     pub started: u32,
     pub paramTypes: u32,
     pub params: [TEEC_Parameter; TEEC_CONFIG_PAYLOAD_REF_COUNT as usize],
-    pub session: *mut TEEC_Session,
+    pub imp: TEEC_Operation__Imp,
 }
 
 extern "C" {
