@@ -15,9 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use crate::raw;
 use core::fmt;
-use hex;
-use optee_teec_sys as raw;
 use uuid as uuid_crate;
 use uuid_crate::parser::ParseError;
 use uuid_crate::BytesError;
@@ -34,7 +33,12 @@ impl Uuid {
     /// # Examples
     ///
     /// ```
-    /// let uuid = Uuid::parse_str("8abcf200-2450-11e4-abe2-0002a5d5c51b").unwrap();
+    /// # use optee_teec::Uuid;
+    /// # use uuid::parser::ParseError;
+    /// # fn main() -> Result<(), ParseError> {
+    /// let uuid = Uuid::parse_str("8abcf200-2450-11e4-abe2-0002a5d5c51b")?;
+    /// # Ok(())
+    /// # }
     /// ```
     pub fn parse_str(input: &str) -> Result<Uuid, ParseError> {
         let uuid = uuid_crate::Uuid::parse_str(input)?;
@@ -52,6 +56,7 @@ impl Uuid {
     /// # Examples
     ///
     /// ```
+    /// # use optee_teec::Uuid;
     /// let bytes: [u8; 16] = [70, 235, 208, 238, 14, 109, 67, 201, 185, 13, 204, 195, 90, 145, 63, 62,];
     /// let uuid = Uuid::from_bytes(bytes);
     /// ```
@@ -66,8 +71,12 @@ impl Uuid {
     /// # Examples
     ///
     /// ```
+    /// # use optee_teec::Uuid;
+    /// # fn main() -> Result<(), uuid::BytesError> {
     /// let bytes: &[u8; 16] = &[70, 235, 208, 238, 14, 109, 67, 201, 185, 13, 204, 195, 90, 145, 63, 62,];
-    /// let uuid = Uuid::from_slice(bytes);
+    /// let uuid = Uuid::from_slice(bytes)?;
+    /// # Ok(())
+    /// # }
     /// ```
     pub fn from_slice(b: &[u8]) -> Result<Uuid, BytesError> {
         let uuid = uuid_crate::Uuid::from_slice(b)?;
@@ -102,6 +111,17 @@ impl Uuid {
     }
 }
 
+impl Clone for Uuid {
+    fn clone(&self) -> Self {
+        Self::new_raw(
+            self.raw.timeLow,
+            self.raw.timeMid,
+            self.raw.timeHiAndVersion,
+            self.raw.clockSeqAndNode,
+        )
+    }
+}
+
 impl fmt::Display for Uuid {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
@@ -129,9 +149,9 @@ mod tests {
             "11173366-2aca-19bc-beb7-10c975e6131e", // random uuid
         ];
         for origin in uuids.iter() {
-            let uuid = Uuid::parse_str(origin).unwrap();
-            let formatted = uuid.to_string();
-            assert_eq!(origin, &formatted);
+            let uuid = Uuid::parse_str(origin);
+            let formatted = uuid.map(|x| x.to_string());
+            assert_eq!(Ok(origin.to_string()), formatted);
         }
     }
 }
