@@ -192,6 +192,11 @@ impl ObjectInfo {
     pub fn object_size(&self) -> usize {
         self.raw.objectSize as usize
     }
+
+    /// Return the `objectType` field of the raw structure `TEE_ObjectInfo`.
+    pub fn object_type(&self) -> u32 {
+        self.raw.objectType
+    }
 }
 
 /// Indicate the possible start offset when moving a data position in the data stream associated with a [PersistentObject](PersistentObject).
@@ -1369,7 +1374,7 @@ impl PersistentObject {
     /// 2) If the Implementation detects any other error associated with this function which is not
     ///    explicitly associated with a defined return code for this function.
     pub fn seek(&self, offset: i32, whence: Whence) -> Result<()> {
-        match unsafe { raw::TEE_SeekObjectData(self.handle(), offset, whence.into()) } {
+        match unsafe { raw::TEE_SeekObjectData(self.handle(), offset.into(), whence.into()) } {
             raw::TEE_SUCCESS => Ok(()),
             code => Err(Error::from_raw_error(code)),
         }
