@@ -79,7 +79,7 @@ impl Uuid {
         ))
     }
 
-    /// Crates a raw TEE client uuid object with specified parameters.
+    /// Creates a raw TEE client uuid object with specified parameters.
     pub fn new_raw(
         time_low: u32,
         time_mid: u16,
@@ -115,6 +115,12 @@ impl fmt::Display for Uuid {
     }
 }
 
+impl From<raw::TEE_UUID> for Uuid {
+    fn from(raw: raw::TEE_UUID) -> Self {
+        Uuid { raw }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     extern crate alloc;
@@ -130,9 +136,9 @@ mod tests {
             "11173366-2aca-19bc-beb7-10c975e6131e", // random uuid
         ];
         for origin in uuids.iter() {
-            let uuid = Uuid::parse_str(origin).unwrap();
-            let formatted = uuid.to_string();
-            assert_eq!(origin, &formatted);
+            let uuid = Uuid::parse_str(origin);
+            let formatted = uuid.map(|x| x.to_string());
+            assert_eq!(Ok(origin.to_string()), formatted);
         }
     }
 }
